'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _bufferLoader = require('../utils/buffer-loader');

var _webAudioScheduler = require('web-audio-scheduler');

var _webAudioScheduler2 = _interopRequireDefault(_webAudioScheduler);

var _noteParser = require('note-parser');

var _noteParser2 = _interopRequireDefault(_noteParser);

var _envelopeGenerator = require('envelope-generator');

var _envelopeGenerator2 = _interopRequireDefault(_envelopeGenerator);

require('./song.css');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; } /* eslint-disable no-loop-func */


var Song = function (_Component) {
  _inherits(Song, _Component);

  function Song(props) {
    _classCallCheck(this, Song);

    var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(Song).call(this, props));

    _this.init = _this.init.bind(_this);
    _this.buffersLoaded = _this.buffersLoaded.bind(_this);
    _this.loop = _this.loop.bind(_this);
    _this.playSample = _this.playSample.bind(_this);
    _this.playSynth = _this.playSynth.bind(_this);
    _this.setupVisualization = _this.setupVisualization.bind(_this);
    _this.createOscillator = _this.createOscillator.bind(_this);
    _this.playToggle = _this.playToggle.bind(_this);
    _this.state = {
      playing: _this.props.autoplay === true
    };
    return _this;
  }

  _createClass(Song, [{
    key: 'componentDidMount',
    value: function componentDidMount() {
      var _this2 = this;

      window.AudioContext = window.AudioContext || window.webkitAudioContext;

      this.context = new AudioContext();

      this.ctx = this.canvas.getContext('2d');

      this.setupVisualization();

      var _props = this.props;
      var tempo = _props.tempo;
      var seqChildren = _props.children;


      var beatInterval = 60000 / tempo;

      this.barInterval = beatInterval * 4;

      var sequencers = _react2.default.Children.toArray(seqChildren);

      this.sequencers = [];

      this.maxBars = 1;

      sequencers.forEach(function (seq) {
        var _seq$props = seq.props;
        var bars = _seq$props.bars;
        var resolution = _seq$props.resolution;
        var children = _seq$props.children;


        if (bars > _this2.maxBars) {
          _this2.maxBars = bars;
        }

        var stepInterval = _this2.barInterval / resolution;
        var instruments = _react2.default.Children.toArray(children);
        _this2.sequencers.push({
          bars: bars,
          stepInterval: stepInterval,
          instruments: instruments
        });
      });

      this.init();
    }
  }, {
    key: 'setupVisualization',
    value: function setupVisualization() {
      var _this3 = this;

      this.visualization = this.context.createScriptProcessor(2048, 1, 1);
      this.visualization.connect(this.context.destination);

      this.analyser = this.context.createAnalyser();
      this.analyser.smoothingTimeConstant = 0.3;
      this.analyser.fftSize = 128;
      this.analyser.connect(this.context.destination);

      var gradient = this.ctx.createLinearGradient(0, 0, 0, 512);
      gradient.addColorStop(1, '#000000');
      gradient.addColorStop(0.75, '#2ecc71');
      gradient.addColorStop(0.25, '#f1c40f');
      gradient.addColorStop(0, '#e74c3c');

      this.visualization.onaudioprocess = function () {
        var array = new Uint8Array(_this3.analyser.frequencyBinCount);
        _this3.analyser.getByteFrequencyData(array);
        _this3.ctx.clearRect(0, 0, 800, 512);
        _this3.ctx.fillStyle = gradient;
        _this3.drawSpectrum(array);
      };
    }
  }, {
    key: 'drawSpectrum',
    value: function drawSpectrum(array) {
      for (var i = 0; i < array.length; i++) {
        var value = array[i];
        this.ctx.fillRect(i * 12, 512, 10, value * -2);
      }
    }
  }, {
    key: 'init',
    value: function init() {
      this.scheduler = new _webAudioScheduler2.default({
        context: this.context
      });

      var sounds = [];

      this.sequencers.forEach(function (seq) {
        seq.instruments.forEach(function (inst) {
          if (inst.type.displayName === 'Sampler') {
            sounds.push(inst.props.sample);
          }
        });
      });

      if (sounds.length) {
        var bufferLoader = new _bufferLoader.BufferLoader(this.context, sounds, this.buffersLoaded);

        bufferLoader.load();
      } else {
        if (this.props.autoplay) {
          this.scheduler.start(this.loop);
        }
      }
    }
  }, {
    key: 'buffersLoaded',
    value: function buffersLoaded(bufferList) {
      this.bufferList = bufferList;
      if (this.props.autoplay) {
        this.scheduler.start(this.loop);
      }
    }
  }, {
    key: 'playSample',
    value: function playSample(e) {
      var source = this.context.createBufferSource();
      source.buffer = e.args.buffer;
      source.detune.value = e.args.detune;
      source.connect(e.args.connectNode);
      source.start(e.playbackTime);
      this.scheduler.nextTick(e.playbackTime + e.args.buffer.duration, function () {
        source.disconnect();
        e.args.connectNode.disconnect();
        e.args.gainNode.disconnect();
      });
    }
  }, {
    key: 'playSynth',
    value: function playSynth(e) {
      var _this4 = this;

      var _e$args = e.args;
      var props = _e$args.props;
      var note = _e$args.note;
      var duration = _e$args.duration;

      if (Array.isArray(note)) {
        note.forEach(function (n) {
          _this4.createOscillator({
            note: n,
            type: props.type,
            connectNode: e.args.connectNode,
            gainNode: e.args.gainNode,
            envelope: props.envelope,
            playbackTime: e.playbackTime,
            duration: duration,
            stepInterval: e.args.stepInterval
          });
        });
      } else {
        this.createOscillator({
          note: note,
          type: props.type,
          connectNode: e.args.connectNode,
          gainNode: e.args.gainNode,
          envelope: props.envelope,
          playbackTime: e.playbackTime,
          duration: duration,
          stepInterval: e.args.stepInterval
        });
      }
    }
  }, {
    key: 'createOscillator',
    value: function createOscillator(options) {
      var note = options.note;
      var type = options.type;
      var connectNode = options.connectNode;
      var gainNode = options.gainNode;
      var envelope = options.envelope;
      var playbackTime = options.playbackTime;
      var duration = options.duration;


      var env = new _envelopeGenerator2.default(this.context, {
        curve: 'exponential',
        attackCurve: 'exponential',
        decayCurve: 'exponential',
        releaseCurve: 'exponential',
        attackTime: envelope.attack,
        decayTime: envelope.decay,
        sustainLevel: envelope.sustain,
        releaseTime: envelope.release
      });
      env.connect(gainNode);

      var oscillator = this.context.createOscillator();
      oscillator.frequency.value = _noteParser2.default.freq(note);
      oscillator.type = type;
      oscillator.connect(connectNode);

      oscillator.start(playbackTime);
      env.start(playbackTime);

      env.release(this.context.currentTime + duration);
      var stopAt = env.getReleaseCompleteTime();

      env.stop(stopAt);
      oscillator.stop(stopAt);

      this.scheduler.nextTick(stopAt, function () {
        oscillator.disconnect();
        gainNode.disconnect();
        connectNode.disconnect();
      });
    }
  }, {
    key: 'loop',
    value: function loop(e) {
      var _this5 = this;

      var sampleIndex = 0;
      this.sequencers.forEach(function (seq) {
        var loopCount = _this5.maxBars / seq.bars;
        seq.instruments.forEach(function (inst) {
          var _loop = function _loop(i) {
            var barOffset = _this5.barInterval * seq.bars * i / 1000;
            inst.props.steps.forEach(function (step, stepIndex) {
              var gainNode = void 0;

              var compressorNode = _this5.context.createDynamicsCompressor();
              var _iteratorNormalCompletion = true;
              var _didIteratorError = false;
              var _iteratorError = undefined;

              try {
                for (var _iterator = Object.keys(inst.props.compressor)[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                  var param = _step.value;

                  compressorNode[param].value = inst.props.compressor[param];
                }
              } catch (err) {
                _didIteratorError = true;
                _iteratorError = err;
              } finally {
                try {
                  if (!_iteratorNormalCompletion && _iterator.return) {
                    _iterator.return();
                  }
                } finally {
                  if (_didIteratorError) {
                    throw _iteratorError;
                  }
                }
              }

              if (Array.isArray(inst.props.volume)) {
                gainNode = _this5.context.createGain();
                gainNode.gain.value = inst.props.volume[stepIndex] / 100;
                gainNode.connect(_this5.analyser);
                compressorNode.connect(gainNode);
              } else {
                gainNode = _this5.context.createGain();
                gainNode.gain.value = inst.props.volume / 100;
                gainNode.connect(_this5.analyser);
                compressorNode.connect(gainNode);
              }

              if (inst.type.displayName === 'Sampler') {
                var time = barOffset + step * seq.stepInterval / 1000;
                _this5.scheduler.insert(e.playbackTime + time, _this5.playSample, {
                  buffer: _this5.bufferList[sampleIndex],
                  connectNode: compressorNode,
                  gainNode: gainNode,
                  envelope: inst.props.envelope,
                  detune: inst.props.detune,
                  stepInterval: seq.stepInterval
                });
              } else {
                var _time = barOffset + step[0] * seq.stepInterval / 1000;
                _this5.scheduler.insert(e.playbackTime + _time, _this5.playSynth, {
                  connectNode: compressorNode,
                  gainNode: gainNode,
                  props: inst.props,
                  duration: step[1] * seq.stepInterval / 1000,
                  note: step[2],
                  stepInterval: seq.stepInterval
                });
              }
            });
          };

          for (var i = 0; i < loopCount; i++) {
            _loop(i);
          }

          if (inst.type.displayName === 'Sampler') {
            sampleIndex++;
          }
        });
      });

      this.scheduler.insert(e.playbackTime + this.barInterval * this.maxBars / 1000, this.loop);
    }
  }, {
    key: 'playToggle',
    value: function playToggle() {
      if (this.state.playing) {
        this.scheduler.stop(true);
      } else {
        this.scheduler.start(this.loop);
      }
      this.setState({
        playing: this.state.playing !== true
      });
    }
  }, {
    key: 'render',
    value: function render() {
      var _this6 = this;

      return _react2.default.createElement(
        'div',
        null,
        _react2.default.createElement('canvas', {
          className: 'react-music-canvas',
          width: 800,
          height: 512,
          ref: function ref(c) {
            _this6.canvas = c;
          }
        }),
        ';',
        _react2.default.createElement(
          'button',
          {
            className: 'react-music-button',
            type: 'button',
            onClick: this.playToggle
          },
          this.state.playing ? 'Stop' : 'Play'
        )
      );
    }
  }]);

  return Song;
}(_react.Component);

exports.default = Song;


Song.propTypes = {
  tempo: _react.PropTypes.number,
  autoplay: _react.PropTypes.bool,
  children: _react.PropTypes.node
};

Song.defaultProps = {
  tempo: 90,
  autoplay: true
};